'''
  This demo explains the exact meaning of the parameters of Gabor functions 
  stored in a .b output files of the mp5 program for Matching Pursuit decomposition of signals.

  Software was written by Jaroslaw Zygierewicz and Magdalena Zieleniewska
  at the Department of Biomedical Physics, Institute of Experimental Physics,
  University of Warsaw (http://brain.fuw.edu.pl).

  Copyright (C) 2014 J. Zygierewicz (jarekz@fuw.edu.pl),
  and M. Zieleniewska (magdalena.zieleniewska@fuw.edu.pl)

  For personnal, educationnal, and research purpose, this software is 
  provided under the GNU GPL (V.3) license: you can redistribute it and/or
  modify it under the terms of the version 3 of the GNU General Public 
  License as published by the Free Software Foundation.
          
  To use this software in commercial application, please contact the authors.
    
  If this code is used for research purpose, the reference [1] should be
  cited in the derived publication.

  MP5demo is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with MP5demo; if not, see <http://www.gnu.org/licenses/>.

  [1] "Spindles in Svarog: framework and software parametrization of EEG transients"
  	  P.J. Durka, U. Malinowska, M. Zieleniewska, C. OReilly, P.T. Rozanski and J. Zygierewicz, 
  	  Submitted to Frontiers in Human Neuroscience, 2014.
'''

import numpy as np
import matplotlib.pyplot as py
import os
from book_reader import *


if __name__ == '__main__':
	# generate signal containing one Gabor

	numberOfSamples   = 512
	samplingFrequency = 256.0
	amplitude         = 9.0
	width             = 128/samplingFrequency  # width in seconds
	frequency         = 30.0                   # frequency in Hz
	position          = 256/samplingFrequency  # position in seconds
	phase             = np.pi/3                # phase

	t      = np.linspace(0,numberOfSamples-1,numberOfSamples)/samplingFrequency
	signal = amplitude*np.exp(-np.pi*((t-position)/width)**2)*np.cos(2*np.pi*frequency*(t-position)+phase)

	# save the signal as floats in a file demo.dat
	# demo.cfg prepared for this file contains information about 
	# numberOfSamples and samplingFrequency as above, plus parameters of the
	# decomposition

	f_name = 'demo'
	a = np.array(signal,'float32')
	f = open(f_name+'.dat','wb')
	a.tofile(f)
	f.close()

	#run mp5 -f demo.set if the file demo_smp.b does not exist'
	if not os.path.isfile(f_name+'_smp.b'):
		os.system('mp5 -f demo.set')

	# after that we read in the results of decomposition

	book = BookImporter(f_name+'_smp.b')

	Eatoms = 0
	reconstruction = np.zeros(len(t))
	for i,booknumber in enumerate(book.atoms):
		for atom in book.atoms[booknumber]:
			frequency = atom['params']['f']*book.fs/2
			amplitude = atom['params']['amplitude']
			phase 	  = atom['params']['phase']
			position  = atom['params']['t']/book.fs
			width     = atom['params']['scale']/book.fs
			reconstruction += amplitude*np.exp(-np.pi*((t-position)/width)**2)*np.cos(2*np.pi*frequency*(t-position)+phase)
			Eatoms += atom['params']['modulus']
	# plotting
	py.subplot(311)
	py.plot(t,signal); 
	py.title('simulated signal from demo.dat')
	py.subplot(312)
	py.plot(t,book.signals[1][0]); 
	py.title('signal stored in demo\_smp.b')
	# this is signal read from the *.b file, should be the same as the original
	py.subplot(313)
	py.plot(t,reconstruction,'r');
	py.title('signal reconstructed from MP decomposition')
	py.show()

	# energy check
	Esignal = sum(book.signals[1][0]**2)
	print 'energy of the signal: ', np.round(Esignal,3)
	print 'energy of atom(s): ', np.round(Eatoms**2,3)